@@ -1,7 +1,7 @@
 /*
  * common.h - Provide global definitions
  *
- * Copyright (C) 2013 - 2016, Max Lv <max.c.lv@gmail.com>
+ * Copyright (C) 2013 - 2017, Max Lv <max.c.lv@gmail.com>
  *
  * This file is part of the shadowsocks-libev.
  * shadowsocks-libev is free software; you can redistribute it and/or modify
@@ -32,6 +32,8 @@
 #define MODULE_LOCAL
 #endif
 
+#include "crypto.h"
+
 int init_udprelay(const char *server_host, const char *server_port,
 #ifdef MODULE_LOCAL
                   const struct sockaddr *remote_addr, const int remote_addr_len,
@@ -39,11 +41,11 @@ int init_udprelay(const char *server_host, const char *server_port,
                   const ss_addr_t tunnel_addr,
 #endif
 #endif
-                  int mtu, int method, int auth, int timeout, const char *iface);
+                  int mtu, crypto_t *crypto, int timeout, const char *iface);
 
 void free_udprelay(void);
 
-#ifdef ANDROID
+#ifdef __ANDROID__
 int protect_socket(int fd);
 int send_traffic_stat(uint64_t tx, uint64_t rx);
 #endif
@@ -53,6 +55,23 @@ int send_traffic_stat(uint64_t tx, uint64_t rx);
 #define STAGE_HANDSHAKE  1  /* Handshake with client            */
 #define STAGE_PARSE      2  /* Parse the header                 */
 #define STAGE_RESOLVE    4  /* Resolve the hostname             */
-#define STAGE_STREAM     5  /* Stream between client and server */
+#define STAGE_WAIT       5  /* Wait for more data               */
+#define STAGE_STREAM     6  /* Stream between client and server */
+
+/* Vals for long options */
+enum {
+    GETOPT_VAL_HELP = 257,
+    GETOPT_VAL_REUSE_PORT,
+    GETOPT_VAL_FAST_OPEN,
+    GETOPT_VAL_ACL,
+    GETOPT_VAL_MTU,
+    GETOPT_VAL_MPTCP,
+    GETOPT_VAL_PLUGIN,
+    GETOPT_VAL_PLUGIN_OPTS,
+    GETOPT_VAL_PASSWORD,
+    GETOPT_VAL_KEY,
+    GETOPT_VAL_MANAGER_ADDRESS,
+    GETOPT_VAL_EXECUTABLE
+};
 
 #endif // _COMMON_H
