@@ -1,7 +1,7 @@
 /*
  * netutils.h - Network utilities
  *
- * Copyright (C) 2013 - 2016, Max Lv <max.c.lv@gmail.com>
+ * Copyright (C) 2013 - 2017, Max Lv <max.c.lv@gmail.com>
  *
  * This file is part of the shadowsocks-libev.
  *
@@ -23,6 +23,8 @@
 #ifndef _NETUTILS_H
 #define _NETUTILS_H
 
+#include <sys/socket.h>
+
 #if defined(__linux__)
 #include <netdb.h>
 #else
@@ -46,17 +48,15 @@
 #endif
 #endif
 
-/* Backward compatibility for MPTCP_ENABLED between kernel 3 & 4 */
+/* MPTCP_ENABLED setsockopt values for kernel 4 & 3, best behaviour to be independant of kernel version is to test from newest to the latest values */
 #ifndef MPTCP_ENABLED
-#ifdef TCP_CC_INFO
-#define MPTCP_ENABLED 42
+static const char mptcp_enabled_values[] = { 42, 26, 0 };
 #else
-#define MPTCP_ENABLED 26
-#endif
+static const char mptcp_enabled_values[] = { MPTCP_ENABLED, 0 };
 #endif
 
 #ifndef UPDATE_INTERVAL
-#define UPDATE_INTERVAL 10
+#define UPDATE_INTERVAL 5
 #endif
 
 /** byte size of ip4 address */
